--[[
	RollTracker Classic 
	1.63
		- track on ItemType
		- change jokers to %joker%
		- option track solo/group/raid<=10/raid>10
		- option ShortMessage
		
	1.62
		- workaround for a bug
		- improvments for loot-tracker
	
	1.61
		- use hooksecurefunc for clearloots
		- include LootTracker and CSV Export
		
		

	based on RollTracker Lite v0.4 - by Jerry Chong. <zanglang@gmail.com>
	Originally written by Coth of Gilneas and Morodan of Khadgar.
	Russian translation by tierggg and Hubbotu
	
	TODO 
		- mehrfilteroptionen (typ)
		- whitelist
		- timer für ausblenden.
		- Undermine Muschelsuppe, Golddorntee, Kodoeintopf - mehrfachoption.
		
]]--

local MAXRARITY=6

local RTC_TOCNAME,RTC = ...

local isDebug=nil

local RTC_Version=GetAddOnMetadata(RTC_TOCNAME, "Version") 
local RTC_Title=GetAddOnMetadata(RTC_TOCNAME, "Title") 

local IconDice= "Interface\\Buttons\\UI-GroupLoot-Dice-Up"
local IconGreed= "Interface\\Buttons\\UI-GroupLoot-Coin-Up"
local IconPass= "Interface\\Buttons\\UI-GroupLoot-Pass-Up"
local IconLoot= "Interface\\GroupFrame\\UI-Group-MasterLooter"
local TxtEscapePicture="|T%s:0|t"

local Tool
local Options 
local MinmapButton

local MSGPREFIX="RTC: "
local MSGPREFIX_START="RTC_StartRoll: "

local rollArray
local rollNames
local rollUndoArray
local rollUndoNames
local lootUndo
--RollTrackerClassicLoot

local L={}

local PatternRoll
local PatternLoot
local PatternLootOwn
local PatternCreateOwn

local PopupDynamic

local LootHistoryHandle
local LootHistoryCountHandle
local LootHistroyFrameHooked
local LootHistoryCleardRollID
local LootHistoryCloseTimer
local LOOTHISTORYCLOSETIMERTIME=5

local CSV_Transfer_Table

local LOOTTRACKER_MAXLINES=1000
--

local RTC_GetPlayerList --()
local RTC_PopupMinimap --(frame)
local RTC_GetAutoChannel --()
local RTC_AddChat --()

-- LootHistory
local RTC_MyLootHistoryFrame_FullUpdate --(self)
local RTC_LootHistoryClear --()
local RTC_LootHistoryShow --(rollID)

-- GUI
local RTC_ResetWindow --()
local RTC_ToggleWindow --(id)
local RTC_MenuButtonClick --(self,button)
local RTC_MenuToolTip --(self)
-- RollTrackerClassic_ResizeButtons()
-- RollTrackerClassic_BtnClose()
-- RollTrackerClassic_BtnRoll()
-- RollTrackerClassic_BtnGreed()
-- RollTrackerClassic_BtnPass()
-- RollTrackerClassic_BtnClearRolls()
-- RollTrackerClassic_BtnAnnounce()
-- RollTrackerClassic_BtnNotRolled()
-- RollTrackerClassic_BtnSettings()
local RTC_SaveAnchors --()
local RTC_ShowWindow --(id)
local RTC_HideWindow --()


-- Option Panel
local RTCO_Update --()
local RTCO_CheckBoxLT --(Var,Init)
local RTCO_CheckBox --(Var,Init)
local RTCO_Init --()

-- Init
local RTC_Init --()

-- Event handler
-- RollTrackerClassic_OnEvent(event, ...)	
-- RollTrackerClassic_OnLoad()
-- RollTrackerClassic_OnDragStart()
-- RollTrackerClassic_OnDragStop()
local RTC_CHAT_MSG_SYSTEM --(arg1)
local RTC_CHAT_MSG_GROUP --(msg,name)

-- Roll
local RTC_AddRoll --(name,roll,low,high)
local RTC_SortRolls --(a, b)
local RTC_FormatRollText --(roll,party,partyName)
local RTC_UpdateRollList --()
local RTC_ClearRolls --()
local RTC_UndoRolls --()
local RTC_NotRolled --()
local RTC_StartRoll --()
local RTC_RollAnnounce --()

-- Loot
local RTC_AddLoot --(name,loot)
local RTC_ClearLoot --()
local RTC_UndoLoot --()
local RTC_LootList_AddItem --(loot)
local RTC_UpdateLootList --()
local RTC_LootListInit --()
-- RollTrackerClassic_ScrollLootList(self,delta)
-- RollTrackerClassic_ScrollCSVList(self,delta)
local RTC_CSVList_AddItem --(loot)
local RTC_CSVListInit --()

---

RTC_GetPlayerList=function()
	local count,start
	local prefix
	local localizedClass, englishClass, classIndex
	local ret={}
	local retName={}
		
	if IsInRaid() then
		prefix="raid"
		count=MAX_RAID_MEMBERS
		start=1
	else
		prefix="party"
		count=MAX_PARTY_MEMBERS
		start=0
	end
		
	for index = start,count do
		local name=""
		if index>0 then
			name=GetUnitName(prefix..index)
			localizedClass, englishClass, classIndex = UnitClass(prefix..index)
		else
			name=GetUnitName("player")
			localizedClass, englishClass, classIndex = UnitClass("player")
		end

		if name~=nil then
			local entry={
				["name"]=name,
				["class"]=englishClass,
				}
			tinsert(ret,entry)
			retName[name]=entry
		end
		
	end
	
	sort(ret,function(a,b) return (a.class<b.class or (a.class==b.class and a.name < b.name)) end)

	return ret,retName
end

RTC_PopupMinimap=function(frame)
	PopupDynamic:Wipe()
	PopupDynamic:AddItem(L["BtnRoll"],nil,false,RollTrackerClassic_BtnRoll)
	if RollTrackerClassicDB.NeedAndGreed then
		PopupDynamic:AddItem(L["BtnGreed"],nil,false,RollTrackerClassic_BtnGreed)
	end
	PopupDynamic:AddItem(L["BtnPass"],nil,false,RollTrackerClassic_BtnPass)
	
	PopupDynamic:AddItem("",nil,true)
	
	PopupDynamic:AddItem(L["BtnOpen"],nil,false,RTC_ShowWindow,1)
	PopupDynamic:AddItem(L["BtnOpenLoot"],nil,false,RTC_ShowWindow,2)	
	PopupDynamic:AddItem(L["BtnLootRolls"],nil,false,RTC_LootHistoryShow)
	PopupDynamic:AddItem(L["BtnCSVEpxort"],nil,false,RTC_ShowWindow,3)	
	PopupDynamic:AddItem(L["BtnConfig"],nil,false,Options.Open,Options)
	
	PopupDynamic:Show(frame)
end

RTC_GetAutoChannel=function ()
	-- Return an appropriate channel in order of preference: /raid, /p, /s
	local channel
	if IsInRaid() then
		channel = "RAID"
	elseif IsInGroup() then
		channel = "PARTY"
	else 
		channel = "SAY"
	end
	return channel
end

RTC_AddChat=function (msg)
	if msg then
		if IsInGroup() or IsInRaid() then
			SendChatMessage(msg , RTC_GetAutoChannel())
		else
			DEFAULT_CHAT_FRAME:AddMessage(msg)
		end
	end
end


-- LootHistory
RTC_MyLootHistoryFrame_FullUpdate= function(self)
	local numItems = C_LootHistory.GetNumItems()	
	for i=1,numItems do
		if self.itemFrames[i] and self.itemFrames[i].rollID and LootHistoryCleardRollID[self.itemFrames[i].rollID]  then
			self.itemFrames[i]:Hide()
		end
	end
end

RTC_LootHistoryClear=function ()
	if LootHistroyFrameHooked==nil then
		LootHistroyFrameHooked=true
		hooksecurefunc("LootHistoryFrame_FullUpdate",RTC_MyLootHistoryFrame_FullUpdate)
	end
	
	local numItems = C_LootHistory.GetNumItems()
	
	for i=1,numItems do
		if LootHistoryFrame.itemFrames[i] then 
			LootHistoryCleardRollID[LootHistoryFrame.itemFrames[i].rollID]=true			
		end
	end
		
	LootHistoryFrame_FullUpdate(LootHistoryFrame)
end

RTC_LootHistoryShow=function(rollID)
	if not LootHistoryFrame:IsShown() or rollID==nil then
		ToggleLootHistoryFrame()
	end
	if rollID then 
		LootHistoryFrame.expandedRolls[rollID] = true
		LootHistoryFrame_FullUpdate(LootHistoryFrame)
	end	
end

-- GUI

RTC_ResetWindow= function()
	RollTrackerClassicMainWindow:ClearAllPoints()
	RollTrackerClassicMainWindow:SetPoint("Center", UIParent, "Center", 0, 0)
	RollTrackerClassicMainWindow:SetWidth(200)
	RollTrackerClassicMainWindow:SetHeight(200)
	RTC_SaveAnchors()
	RTC_ShowWindow(1)
end

RTC_ToggleWindow= function(id)
	if RollTrackerClassicMainWindow:IsVisible() then
		RTC_HideWindow()
	else
		RTC_ShowWindow(id)
	end
end

RTC_MenuButtonClick=function(self,button)
	if button=="LeftButton" then 
		if IsShiftKeyDown() then
			RTC_LootHistoryShow()
		else
			RTC_ToggleWindow()
		end
		
	else
		if self.isMinimapButton then 
			RTC_PopupMinimap(self.button)
		else
			RTC_PopupMinimap(self)
		end
	end
end

RTC_MenuToolTip=function(self)
	self:AddLine(RTC_Title)
	self:AddLine(L["MsgTooltip"])
end	

function RollTrackerClassic_ResizeButtons()
	local w = RollTrackerClassicFrameHelperButton:GetWidth()
	if RollTrackerClassicDB.NeedAndGreed then
		RollTrackerClassicFrameRollButton:SetWidth(w/3)
		RollTrackerClassicFramePassButton:SetWidth(w/3)
		RollTrackerClassicFrameGreedButton:Show()
	else
		RollTrackerClassicFrameRollButton:SetWidth(w/2)
		RollTrackerClassicFramePassButton:SetWidth(w/2)
		RollTrackerClassicFrameGreedButton:Hide()
	end
	
	if RollTrackerClassicDB.ShowNotRolled then
		RollTrackerClassicFrameAnnounceButton:SetWidth(w/3)
		RollTrackerClassicFrameClearButton:SetWidth(w/3)
		RollTrackerClassicFrameNotRolledButton:Show()
	else
		RollTrackerClassicFrameAnnounceButton:SetWidth(w/2)
		RollTrackerClassicFrameClearButton:SetWidth(w/2)
		RollTrackerClassicFrameNotRolledButton:Hide()
	end
	
end

function RollTrackerClassic_BtnClose()
	RTC_HideWindow()
	if RollTrackerClassicDB.ClearOnClose then
		RTC_ClearRolls()
	end
end

function RollTrackerClassic_BtnRoll()
	RandomRoll(1, 100)
end

function RollTrackerClassic_BtnGreed()
	RandomRoll(1, 50)
end

function RollTrackerClassic_BtnPass()
	RTC_AddChat("pass")
end

function RollTrackerClassic_BtnClearRolls()
	if #rollArray>0 then 
		RTC_ClearRolls()
		if RollTrackerClassicDB.CloseOnClear then
			RTC_HideWindow()
		end
	elseif #rollUndoArray>0 then
		RTC_UndoRolls()
	end
end

function RollTrackerClassic_BtnAnnounce()
	RTC_RollAnnounce()
	if RollTrackerClassicDB.ClearOnAnnounce then
		RTC_ClearRolls()
	end
	if RollTrackerClassicDB.CloseOnAnnounce then
		RTC_HideWindow()
	end
end

function RollTrackerClassic_BtnNotRolled()
	RTC_NotRolled()
end

function RollTrackerClassic_BtnSettings()
	if Tool.GetSelectedTab(RollTrackerClassicMainWindow)==1 then 
		Options:Open()
	else
		Options:Open(2)
	end
end

function RollTrackerClassic_BtnLootClear()
	if #RollTrackerClassicLoot>0 then
		RTC_ClearLoot()
	elseif #lootUndo>0 then
		RTC_UndoLoot()
	end
end

RTC_SaveAnchors=function ()
	RollTrackerClassicDB.X = RollTrackerClassicMainWindow:GetLeft()
	RollTrackerClassicDB.Y = RollTrackerClassicMainWindow:GetTop()
	RollTrackerClassicDB.Width = RollTrackerClassicMainWindow:GetWidth()
	RollTrackerClassicDB.Height = RollTrackerClassicMainWindow:GetHeight()
end

RTC_ShowWindow=function (id)
	-- workaround for a strange bug
	if RollTrackerClassicMainWindow.Tabs then 
		for i=1, RollTrackerClassicMainWindow.numTabs do
			local child=RollTrackerClassicMainWindow.Tabs[i].content
			child:SetParent(RollTrackerClassicMainWindow)
			child:ClearAllPoints()
			child:SetPoint("TOPLEFT",8,-30)
			child:SetPoint("BOTTOMRIGHT",-8,8)			
		end
	end

	RollTrackerClassicMainWindow:Show()
	RTC_UpdateRollList()
	RTC_UpdateLootList()
	Tool.SelectTab(RollTrackerClassicMainWindow,id)
end

RTC_HideWindow=function ()
	RollTrackerClassicMainWindow:Hide()
end
	
-- Options Panel
RTCO_Update=function()
	RTC_UpdateRollList()
	RTC_UpdateLootList()
	RTC_LootListInit()
	RTC_CSVListInit()
	RollTrackerClassic_ResizeButtons()
	MinmapButton:UpdatePosition()
end

RTCO_CheckBox= function (Var,Init)
	Options:AddCheckBox(RollTrackerClassicDB,Var,Init,L["Cbox"..Var])
end
RTCO_CheckBoxLT= function (Var,Init)
	Options:AddCheckBox(RollTrackerClassicDB.LootTracker,Var,Init,L["CboxLT"..Var])
end

RTCO_Init=function()
	Options=Lib_GPI.Options("RTCO_",
		function()-- doOk
			Options:DoOk() 
			RTCO_Update()
		end,
		function()-- doCancel
			Options:DoCancel() 
			RTCO_Update()
		end,
		function()--doDefault
			Options:DoDefault() 
			RollTrackerClassicDB.Minimap.position=50
			RTC_ResetWindow()
			
			MinmapButton:UpdatePosition()
			
			RTCO_Update()
		end)

	Options:SetScale(0.85)
	Options:AddPanel(RTC_Title)
	Options:AddVersion('|cff00c0ff' .. RTC_Version .. "|r")
	
	Options:AddCategory( L["HeaderSettings"] ) 
	Options:Indent()
	
	Options:AddCheckBox(RollTrackerClassicDB.Minimap,"visible",true,L["Cboxshowminimapbutton"])
	Options:AddCheckBox(RollTrackerClassicDB.Minimap,"lock",false,L["CboxLockMinimapButton"])
	Options:AddCheckBox(RollTrackerClassicDB.Minimap,"lockDistance",false,L["CboxLockMinimapButtonDistance"])
	Options:AddSpace()
	RTCO_CheckBox("ClearOnClose",true)	
	RTCO_CheckBox("ClearOnAnnounce", true)	
	RTCO_CheckBox("CloseOnAnnounce", true)	
	RTCO_CheckBox("CloseOnClear", true)
	Options:AddSpace()
	RTCO_CheckBox("IgnoreDouble", false)
	RTCO_CheckBox("RejectOutBounds", false)	
	RTCO_CheckBox("AnnounceIgnoreDouble", true)	
	RTCO_CheckBox("AnnounceRejectOutBounds",false)
	Options:AddSpace()
	RTCO_CheckBox("NeedAndGreed",false)
	RTCO_CheckBox("ShowNotRolled",true)
	Options:AddSpace()
	RTCO_CheckBox("ClearOnStart",false)
	RTCO_CheckBox("OpenOnStart",false)
	Options:AddSpace()
	RTCO_CheckBox("ColorName",true)
	RTCO_CheckBox("ShowClassIcon",true)
	Options:AddSpace()
	RTCO_CheckBox("AutoLootRolls",true)
	RTCO_CheckBox("AutoCloseLootRolls",true)
	
	-- loot tracker
	Options:AddPanel(L["PanelLootTracker"],false,true)
	
	Options:AddCategory( L["HeaderSettings"] ) 
	Options:Indent()
	RTCO_CheckBoxLT("ShortMessage",false)
	RTCO_CheckBoxLT("TrackSolo",true)
	RTCO_CheckBoxLT("TrackGroup",true)
	RTCO_CheckBoxLT("TrackSRaid",true)
	RTCO_CheckBoxLT("TrackBRaid",true)
	
	
	Options:Indent(-10)
	
	Options:AddCategory(L["HeaderRarity"])
	Options:Indent(10)
	for i=0,tonumber(MAXRARITY) do
		Options:AddCheckBox(RollTrackerClassicDB.LootTracker.Rarity,i,i>=2,
							ITEM_QUALITY_COLORS[i].hex .._G["ITEM_QUALITY"..i.."_DESC"].."|r")
	end
	Options:Indent(-10)
	
	Options:AddCategory(L["HeaderItemType"])
	Options:Indent()
	Options:AddText(L["TxtItemType"])
	for i = 0, NUM_LE_ITEM_CLASSS-1 do
		local txt=GetItemClassInfo(i)
		if txt~=nil and string.find(txt,"OBSOLETE")==nil then
			Options:AddCheckBox(RollTrackerClassicDB.LootTracker.ItemType,i,false,txt)
		end
	end
	Options:Indent(-10)
	
	Options:AddCategory(L["HeaderCSV"])
	Options:Indent(10)
	Options:AddText(L["TxtCSVJokersTitle"])
	Options:AddText(L["TxtCSVJokers"])
	Options:AddText(L["TxtCSVJokers2"])
	Options:AddSpace()
	--lib:AddEditBox(DB,Var,Init,TXTLeft,width,widthLeft,onlynumbers,tooltip,suggestion)
	Options:AddEditBox(RollTrackerClassicDB.LootTracker,
						"CSVexport",
						'%yy%-%mm%-%dd%;%HH%:%MM%;%name%;%iid%;%icount%',
						L["EdtCSVexport"],
						400)
	
	Options:Indent(-10)
	
	-- localization
	Options:SetScale(0.85)
	Options:AddPanel(L["HeaderCustomLocales"],false,true)
	Options:AddText(L["MsgLocalRestart"])
	Options:AddSpace()
	
	local locales=getmetatable(L).__index
	local t={}
	for key,value in pairs(locales) do 
		table.insert(t,key)
	end
	table.sort(t)
	for i,key in ipairs(t) do 
		local col,txt
		if locales[key]==L[key] then
			col="|cffff4040"
		else
			col="|cffffffff"
		end
		if L[key.."_org"] then
			txt=L[key.."_org"]			
		else
			txt=L[key]
		end
		Options:AddEditBox(RollTrackerClassicDB.CustomLocales,key,"",col.."["..key.."]",450,200,false,locales[key],txt)

		
	end
	
	
end

-- Init
RTC_Init=function ()
	Tool=Lib_GPI.ToolBox()
	L=RTC.GetLocale()
	
	CSV_Transfer_Table={}

	rollArray = {}
	rollNames = {}
	rollUndoArray = {}
	rollUndoNames = {}
	lootUndo={}
	
	LootHistoryCloseTimer=0	
	LootHistoryCountHandle=0
	LootHistoryHandle={}
	LootHistoryCleardRollID={}
	
	-- using strings from GlobalStrings.lua
	PatternRoll = Tool.CreatePattern(RANDOM_ROLL_RESULT)
	PatternLoot = Tool.CreatePattern(LOOT_ITEM,true)
	PatternLootOwn = Tool.CreatePattern(LOOT_ITEM_SELF,true)
	PatternCreateOwn = Tool.CreatePattern(LOOT_ITEM_CREATED_SELF ,true)
	
	-- settings
	if not RollTrackerClassicDB then RollTrackerClassicDB = {} end -- fresh DB	
	if not RollTrackerClassicDB.Minimap then RollTrackerClassicDB.Minimap = {} end 
	if not RollTrackerClassicDB.CustomLocales then RollTrackerClassicDB.CustomLocales = {} end 
	if not RollTrackerClassicLoot then RollTrackerClassicLoot = {} end
	if not RollTrackerClassicDB.LootTracker then RollTrackerClassicDB.LootTracker  = {} end
	if not RollTrackerClassicDB.LootTracker.Rarity then RollTrackerClassicDB.LootTracker.Rarity  = {} end
	if not RollTrackerClassicDB.LootTracker.ItemType then RollTrackerClassicDB.LootTracker.ItemType  = {} end
	
	local x, y, w, h = RollTrackerClassicDB.X, RollTrackerClassicDB.Y, RollTrackerClassicDB.Width, RollTrackerClassicDB.Height
	if not x or not y or not w or not h then
		RTC_SaveAnchors()
	else
		RollTrackerClassicMainWindow:ClearAllPoints()
		RollTrackerClassicMainWindow:SetPoint("TOPLEFT", UIParent, "BOTTOMLEFT", x, y)
		RollTrackerClassicMainWindow:SetWidth(w)
		RollTrackerClassicMainWindow:SetHeight(h)
	end
	
	-- Language
	RollTrackerClassicMainWindowTitle:SetText(string.format(TxtEscapePicture,IconDice).. RTC_Title)
	
	RollTrackerClassicFrameRollButton:SetText(string.format(TxtEscapePicture,IconDice).. L["BtnRoll"])
	RollTrackerClassicFrameGreedButton:SetText(string.format(TxtEscapePicture,IconGreed)..L["BtnGreed"])
	RollTrackerClassicFramePassButton:SetText(string.format(TxtEscapePicture,IconPass)..L["BtnPass"])
	
	--RollTrackerClassicFrameClearButton:SetText(L["BtnClear"])	
	RollTrackerClassicFrameAnnounceButton:SetText(L["BtnAnnounce"])
	RollTrackerClassicFrameNotRolledButton:SetText(L["BtnNotRolled"])
	RollTrackerClassicFrameHelperButton:Hide()
	RollTrackerClassic_ResizeButtons()
	
	RollTrackerClassicLootFrameClearButton:SetText(L["BtnClear"])	
	
	-- Options
	RTCO_Init()
	
	-- slash command
	SLASH_ROLLTRACKERCLASSIC1 = "/rolltracker";
	SLASH_ROLLTRACKERCLASSIC2 = "/rt";
	SLASH_ROLLTRACKERCLASSIC3 = "/rtc";
	SlashCmdList["ROLLTRACKERCLASSIC"] = function (msg)
		if     msg == "clear" or msg=="clear rolls" then
			RTC_ClearRolls(true)
			
		elseif msg == "clear loot" then
			RTC_ClearLoot(true)
			
		elseif msg == "undo" or msg=="undo rolls" then
			RTC_UndoRolls(true)
			RTC_ShowWindow(1)
		
		elseif msg == "undo loot"then
			RTC_UndoLoot(true)
			RTC_ShowWindow(2)	
			
		elseif msg == "announce" then
			RTC_RollAnnounce()
			
		elseif msg == "notrolled" then
			RTC_NotRolled()
			
		elseif msg == "close" then
			RTC_HideWindow()	
			
		elseif msg == "config" or msg=="setup" then
		    Options.Open()
			
		elseif msg == "reset" then
			RTC_ResetWindow()
			
		elseif string.sub(msg,1,5) == "debug" then 
			for name, roll, low, high in string.gmatch(msg,"debug (.+) (%d+) (%d+) (%d+)" ) do
				RTC_AddRoll(name,roll,low,high)
			end
			
		elseif msg=="start" then
			RTC_StartRoll()
			
		elseif msg:match"|c(.+)|H(.+)|h(.+)|h|r" then
			RTC_StartRoll(msg)
		
		elseif msg=="clearlootrolls" then
			RTC_LootHistoryClear()
			
		else
			RTC_ShowWindow()
		end
	end
	
	--gui
	MinmapButton=Lib_GPI.MinimapButton(RollTrackerClassicDB.Minimap,IconDice,RTC_MenuButtonClick,RTC_Title.."\n"..L["MsgTooltipLeft"].."\n"..L["MsgTooltipShiftLeft"].."\n"..L["MsgTooltipRight"])	
	
	Tool.EnableSize(RollTrackerClassicMainWindow,8,"Interface\\AddOns\\RollTrackerClassic\\Resize\\",nil,function()	
		RTC_SaveAnchors()
		end
	)
	
	Tool.AddTab(RollTrackerClassicMainWindow,L["TabRoll"],RollTrackerClassicFrame)
	Tool.AddTab(RollTrackerClassicMainWindow,L["TabLoot"],RollTrackerClassicLootFrame)
	Tool.AddTab(RollTrackerClassicMainWindow,L["TabCSV"],RollTrackerClassicCSVFrame)
	
	PopupDynamic=Tool.CreatePopup()
		
	-- timer
	local FakeTimer=CreateFrame("Frame")
	FakeTimer:SetScript("OnUpdate",
		function()
			if LootHistoryCloseTimer>0 and LootHistoryCloseTimer<time() and LootHistoryCountHandle<=0 then
				if RollTrackerClassicDB.AutoCloseLootRolls then
					LootHistoryFrame_CollapseAll(LootHistoryFrame)
					LootHistoryFrame:Hide()
				end
				LootHistoryCloseTimer=0
				LootHistoryCountHandle=0
			end
		end
		)

	
	
		
	RTC_LootListInit()
	RTC_UpdateLootList()
	RTC_CSVListInit()
end

-- Event handler
function RollTrackerClassic_OnEvent(event, ...)	
	local arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg, arg9 = ...
	--print("rtc-event:"..event.." "..arg1)
	if  event == "ADDON_LOADED" then
		if arg1 == RTC_TOCNAME then
			RTC_Init()
		end
		if Tool then 
			Tool:AddDataBrocker(IconDice, RTC_MenuButtonClick, RTC_MenuToolTip, RTC_TOCNAME, RTC_Title)
		end
		
	elseif event=="START_LOOT_ROLL" then
		--START_LOOT_ROLL: rollID, rollTime, lootHandle
		if RollTrackerClassicDB.AutoLootRolls then
			RTC_LootHistoryShow(arg1)
		end
		LootHistoryHandle[arg3]=true
		LootHistoryCountHandle=LootHistoryCountHandle+1
		LootHistoryCloseTimer=0	
		
	elseif event=="LOOT_ROLLS_COMPLETE" then
		--LOOT_ROLLS_COMPLETE: lootHandle
		if LootHistoryHandle[arg1]==true then
			LootHistoryHandle[arg1]=nil
			LootHistoryCountHandle=LootHistoryCountHandle-1
			if LootHistoryCountHandle==0 then
				LootHistoryCloseTimer=time()+LOOTHISTORYCLOSETIMERTIME				
			end	
		end

	
	elseif (event == "CHAT_MSG_LOOT" or event == "CHAT_MSG_SYSTEM") then
		RTC_CHAT_MSG_SYSTEM(arg1)
		
	elseif (event == "CHAT_MSG_PARTY" or event == "CHAT_MSG_PARTY_LEADER" or event == "CHAT_MSG_RAID" or event == "CHAT_MSG_RAID_LEADER") then
		RTC_CHAT_MSG_GROUP(arg1,arg2)
	end
end

function RollTrackerClassic_OnLoad()
    --print("rtc-onload")
	RollTrackerClassicMainWindow:SetMinResize(194,170)
	RollTrackerClassicMainWindow:RegisterForDrag("LeftButton")
	RollTrackerClassicMainWindow:RegisterEvent("ADDON_LOADED")
	RollTrackerClassicMainWindow:RegisterEvent("CHAT_MSG_SYSTEM")
	RollTrackerClassicMainWindow:RegisterEvent("CHAT_MSG_PARTY")
	RollTrackerClassicMainWindow:RegisterEvent("CHAT_MSG_PARTY_LEADER")
	RollTrackerClassicMainWindow:RegisterEvent("CHAT_MSG_RAID")
	RollTrackerClassicMainWindow:RegisterEvent("CHAT_MSG_RAID_LEADER")
	RollTrackerClassicMainWindow:RegisterEvent("START_LOOT_ROLL")
	RollTrackerClassicMainWindow:RegisterEvent("LOOT_ROLLS_COMPLETE")
	RollTrackerClassicMainWindow:RegisterEvent("CHAT_MSG_LOOT")
end

function RollTrackerClassic_OnDragStart()
	RollTrackerClassicMainWindow:StartMoving()
end

function RollTrackerClassic_OnDragStop()
	RollTrackerClassicMainWindow:StopMovingOrSizing();
	RTC_SaveAnchors()
end

RTC_CHAT_MSG_SYSTEM=function (arg1)
	for name, roll, low, high in string.gmatch(arg1, PatternRoll) do
		--print(".."..name.." "..roll.." "..low.." "..high)		
		RTC_AddRoll(name,roll,low,high)		
	end
	-- %s receives loot : %s|Hitem :%d :%d :%d :%d|h[%s]|h%s. 
	for name, item in string.gmatch(arg1, PatternLoot) do	
		RTC_AddLoot(name,item)
	end
	--You receive loot : %s|Hitem :%d :%d :%d :%d|h[%s]|h%s.
	
	for item,b,c,d in string.gmatch(arg1, PatternLootOwn) do
		RTC_AddLoot(GetUnitName("player"),item)
	end	
	-- You create : %s|Hitem :%d :%d :%d :%d|h[%s]|h%s. 
	--for item,b,c,d in string.gmatch(arg1, PatternCreateOwn) do
	--	RTC_AddLoot(GetUnitName("player"),item)
	--end
end

RTC_CHAT_MSG_GROUP=function (msg,name)
	if msg == L["pass"] or msg == "pass" then
		name=Tool.Split(name, "-")[1]
		RTC_AddRoll(name,"0","1","100")
	end	
	
	if string.sub(msg,1,string.len(MSGPREFIX_START))==MSGPREFIX_START then
		if RollTrackerClassicDB.ClearOnStart then 
			RTC_ClearRolls()
		end
		if RollTrackerClassicDB.OpenOnStart then 
			RTC_ShowWindow(1)
		end
	end
	
	
end

-- Roll 

RTC_AddRoll=function (name,roll,low,high)
	local ok=false
	-- check for rerolls. >1 if rolled before
	if RollTrackerClassicDB.NeedAndGreed then
		if (RollTrackerClassicDB.IgnoreDouble==false or rollNames[name]==nil or rollNames[name]==0) and
		   ((low=="1" and high=="50") or (low=="1" and high=="100")) then
			ok=true
		end
	else
		if (RollTrackerClassicDB.IgnoreDouble==false or rollNames[name]==nil or rollNames[name]==0) and
		   (RollTrackerClassicDB.RejectOutBounds==false or (low=="1" and high=="100")) then
			ok=true
		end
	end
	
	if ok then
		rollNames[name] = rollNames[name] and rollNames[name] + 1 or 1
		table.insert(rollArray, {
			Name = name,
			Roll = tonumber(roll),
			Low = tonumber(low),
			High = tonumber(high),
			Count = rollNames[name]
		})
		
		RTC_ShowWindow(1)
		
	else	
		DEFAULT_CHAT_FRAME:AddMessage(string.format(MSGPREFIX .. L["MsgCheat"],name,roll,low,high))
	end
end

RTC_SortRolls= function (a, b)
	return a.Roll < b.Roll
end

RTC_FormatRollText=function (roll,party,partyName)
	local colorTied= "|cffffffff"
	local colorCheat=((roll.Low ~= 1 or roll.High ~= 100) or (roll.Count > 1)) and  "|cffffcccc" or "|cffffffff"
	local txtRange=(roll.Low ~= 1 or roll.High ~= 100) and format(" (%d-%d)", roll.Low, roll.High)  or ""
	
	local colorName
	local iconClass
	
	if partyName[roll.Name] and partyName[roll.Name].class then
		colorName="|c"..RAID_CLASS_COLORS[partyName[roll.Name].class].colorStr
		iconClass=Lib_GPI.IconClass[partyName[roll.Name].class]
	end
	if colorName==nil or RollTrackerClassicDB.ColorName==false then colorName=colorCheat end
	if iconClass==nil or RollTrackerClassicDB.ShowClassIcon==false then iconClass=""	end
	
	local txtCount=roll.Count > 1 and format(" [%d]", roll.Count) or ""
	
	return colorTied..string.format("%3d",roll.Roll).."|r: "..
			iconClass..colorName.. roll.Name.."|r "..
			colorCheat..txtRange.."|r "..
			colorCheat..txtCount.."\n"

end

RTC_UpdateRollList=function ()
	local rollText = ""
	
	local party,partyName = RTC_GetPlayerList()
	
	table.sort(rollArray, RTC_SortRolls)
	
	-- format and print rolls, check for ties
	if RollTrackerClassicDB.NeedAndGreed then
		local rtxt=""
		for i, roll in pairs(rollArray) do
			if roll.Roll>0 and roll.High == 100 then
				rtxt=RTC_FormatRollText(roll,party,partyName)..rtxt
			end
		end
		rollText = "|cFFAAAAAA" .. L["TxtNeed"] .."\n" .. rtxt 		
		rtxt=""
		for i, roll in pairs(rollArray) do
			if roll.Roll==0 or roll.High == 50 then
				rtxt=RTC_FormatRollText(roll,party,partyName)..rtxt
			end			
		end
		rollText = rollText .. "\n|cFFAAAAAA" .. L["TxtGreed"] .. "\n" .. rtxt
	else
		for i, roll in pairs(rollArray) do
			rollText=RTC_FormatRollText(roll,party,partyName) .. rollText
		end
	end

	--if IsInGroup() or IsInRaid() then
	rollText = rollText.."|cFFAAAAAA" .. L["TxtLine"] .. "\n"
	local gtxt=""
	local missClasses={}
	for i,p in ipairs(party) do 
		if rollNames[p.name]==nil or rollNames[p.name]==0 then
			local iconClass=Lib_GPI.IconClass[partyName[p.name].class]
			if iconClass==nil or RollTrackerClassicDB.ShowClassIcon==false then 
				iconClass=""	
			else
				missClasses[partyName[p.name].class]=missClasses[partyName[p.name].class] and missClasses[partyName[p.name].class]+1 or 1
			end
			gtxt = gtxt.. iconClass .. p.name .."\n"
		end
	end
	local ctxt=""
	if IsInRaid() then 
		for class,count in pairs(missClasses) do
			ctxt=ctxt..Lib_GPI.IconClass[class]..count.." "
		end
		if ctxt~="" then ctxt=ctxt.."\n" .. L["TxtLine"] .. "\n" end
	end
	
	rollText=rollText..ctxt..gtxt

	
	--end
	
	RollTrackerRollText:SetText(rollText)
	RollTrackerClassicFrameStatusText:SetText(string.format(L["MsgNbRolls"], table.getn(rollArray)))

	if #rollArray==0 and #rollUndoArray>0 then
		RollTrackerClassicFrameClearButton:SetText(L["BtnUndo"])
	else
		RollTrackerClassicFrameClearButton:SetText(L["BtnClear"])	
	end
	
end

RTC_ClearRolls=function (doMsg)
	if #rollArray>0 then
		if doMsg then
			DEFAULT_CHAT_FRAME:AddMessage(MSGPREFIX .. L["MsgRollCleared"])
		end
	
		rollUndoArray = rollArray
		rollUndoNames = rollNames
		rollArray = {}
		rollNames = {}			
	end
	RTC_UpdateRollList()
end

RTC_UndoRolls=function (doMsg)
	if #rollUndoArray>0 then
		if doMsg then
			DEFAULT_CHAT_FRAME:AddMessage(MSGPREFIX .. L["MsgUndoRoll"])
		end
		rollArray = rollUndoArray
		rollNames = rollUndoNames
		rollUndoArray = {}
		rollUndoNames = {}	
		RTC_UpdateRollList()
	end
end

RTC_NotRolled=function ()
	if IsInGroup() or IsInRaid() then
	
		local party,partyName = RTC_GetPlayerList()
		local names=""
		for i,p in ipairs(party) do 
			if rollNames[p.name]==nil or rollNames[p.name]==0 then
				names = names ..", ".. p.name
			end
		end
		names=string.sub(names,3)
		
		if names ~= "" then 
			RTC_AddChat(MSGPREFIX .. string.format(L["MsgNotRolled"] ,L["pass"]))
			RTC_AddChat(names)							
		end
		
	end
end

RTC_StartRoll=function (item)
	local msg
	RTC_ShowWindow(1)
	RTC_ClearRolls()
	if RollTrackerClassicDB.NeedAndGreed then
		msg=L["MsgStartGreenAndNeed"]
	else
		msg=L["MsgStart"]
	end
	RTC_AddChat(MSGPREFIX_START .. string.format(msg,L["pass"]))
	if item then
		RTC_AddChat(MSGPREFIX .. string.format(L["MsgNextItem"],item))
	end
	RTC_AddChat(L["MsgBar"])
end	

RTC_RollAnnounce=function ()
	local winNum=0
	local winName=""
	local max=-1
	local addPrefix=""
	local msg=""
	if RollTrackerClassicDB.NeedAndGreed then
		for i, roll in pairs(rollArray) do		
			if (RollTrackerClassicDB.AnnounceIgnoreDouble==false or roll.Count==1) and
			  (roll.Roll>0 and roll.Low==1 and roll.High==100) then
		
				if roll.Roll==max then
					winNum=winNum+1
					winName=winName..", "..roll.Name
				elseif roll.Roll>max then
					max=roll.Roll
					winNum=1
					winName=roll.Name
				end
			end
		end
		
		if winNum==0 then
			for i, roll in pairs(rollArray) do		
				if (RollTrackerClassicDB.AnnounceIgnoreDouble==false or roll.Count==1) and
				  (roll.Roll == 0 or ( roll.Low==1 and roll.High==50)) then
			
					if roll.Roll==max then
						winNum=winNum+1
						winName=winName..", "..roll.Name
					elseif roll.Roll>max then
						max=roll.Roll
						winNum=1
						winName=roll.Name
					end
				end
			end
			addPrefix=L["TxtGreed"].."! "
		else
			addPrefix=L["TxtNeed"].."! "
		end
		
	else
		for i, roll in pairs(rollArray) do
		
			if (RollTrackerClassicDB.AnnounceIgnoreDouble==false or roll.Count==1) and
			  (RollTrackerClassicDB.AnnounceRejectOutBounds==false or (roll.Low==1 and roll.High==100)) then
		
				if roll.Roll==max then
					winNum=winNum+1
					winName=winName..", "..roll.Name
				elseif roll.Roll>max then
					max=roll.Roll
					winNum=1
					winName=roll.Name
				end
			end
		end
	end
	
	if winNum==1 then 
		msg=MSGPREFIX .. addPrefix .. string.format(L["MsgAnnounce"] ,winName,max) 
	elseif winNum>1 then
		msg=MSGPREFIX .. addPrefix .. string.format(L["MsgAnnounceTie"] ,winName,max) 
	end
	
	RTC_AddChat(msg)

end

-- Loot

RTC_AddLoot=function (name,lootitem)
	if name==nil or lootitem==nil then return end
	
	local ok=false
	if IsInRaid() then
		local count=GetNumGroupMembers()
		if RollTrackerClassicDB.LootTracker.TrackSRaid and count<=10 then
			ok=true
		elseif RollTrackerClassicDB.LootTracker.TrackBRaid and count>=11 then
			ok=true
		end
	elseif IsInGroup() then
		if RollTrackerClassicDB.LootTracker.TrackGroup then
			ok=true
		end
	elseif RollTrackerClassicDB.LootTracker.TrackSolo then
		ok=true
	end
	
	if not ok then return end
	
	local _,PartyNames = RTC_GetPlayerList()
	local t=time()
	local class
	if PartyNames[name] then
		class=PartyNames[name].class
	end
	
	local loot={
		["name"]=name,
		["class"]=class,
		["timestamp"]=t,
		}
	RollTrackerClassicDB.DEBUGHELP_ITEM=lootitem
	print(lootitem)
	
	loot.itemName, loot.itemLink, loot.itemRarity, loot.itemLevel, _, _,
		_, _, _, _, loot.itemSellPrice,loot.itemType =GetItemInfo(string.match(lootitem,"|H(.+)%["))
	
	loot.ItemCount=tonumber(string.match(lootitem,"|rx(%d+)") or 1)

	
	if RollTrackerClassicDB.LootTracker.Rarity[loot.itemRarity] or 
			RollTrackerClassicDB.LootTracker.ItemType[loot.itemType] then
		
		tinsert(RollTrackerClassicLoot,loot)	
		RTC_CSVList_AddItem(loot)
		RTC_LootList_AddItem(loot)
		RTC_UpdateLootList()		
	end
	
end

RTC_ClearLoot=function(doMsg)
	if #RollTrackerClassicLoot>0 then
		if doMsg then
			DEFAULT_CHAT_FRAME:AddMessage(MSGPREFIX .. L["MsgLootCleared"])
		end
		lootUndo = RollTrackerClassicLoot
		RollTrackerClassicLoot={}
		RTCO_Update()
	end
end

RTC_UndoLoot=function (doMsg)
	if #lootUndo>0 then
		if doMsg then
			DEFAULT_CHAT_FRAME:AddMessage(MSGPREFIX .. L["MsgUndoLoot"])
		end
		RollTrackerClassicLoot = lootUndo
		lootUndo = {}	
		RTCO_Update()
	end
end

RTC_LootList_AddItem=function (loot)
	local colorName
	local iconClass
	if loot.class then
		colorName="|c"..RAID_CLASS_COLORS[loot.class].colorStr
		iconClass=Lib_GPI.IconClass[loot.class]
	end
	if colorName==nil or RollTrackerClassicDB.ColorName==false then colorName="" end
	if iconClass==nil or RollTrackerClassicDB.ShowClassIcon==false then iconClass="" end
	
	local stack
	if loot.ItemCount~=nil and loot.ItemCount>1 then
		stack="x"..loot.ItemCount
	else
		stack=""
	end	
	
	local Text
	if RollTrackerClassicDB.LootTracker.ShortMessage then
		Text=iconClass..colorName.. loot.name.."|r".." "..loot.itemLink..stack
	else
		Text=string.format(L["MsgLootLine"],
					date("%y-%m-%d %H:%M",loot.timestamp),
					iconClass..colorName.. loot.name.."|r",
					loot.itemLink..stack
					)
	end
	RollTrackerClassicLootFrame_MessageFrame:AddMessage( Text,.8,.8,.8)
end

RTC_LootListInit=function()

	RollTrackerClassicLootFrame_MessageFrame:SetFading(false);
	RollTrackerClassicLootFrame_MessageFrame:SetFontObject(GameFontNormalSmall);
	RollTrackerClassicLootFrame_MessageFrame:SetJustifyH("LEFT");
	RollTrackerClassicLootFrame_MessageFrame:SetHyperlinksEnabled(true);
	RollTrackerClassicLootFrame_MessageFrame:SetTextCopyable(true);
	RollTrackerClassicLootFrame_MessageFrame:Clear()
	RollTrackerClassicLootFrame_MessageFrame:SetMaxLines(LOOTTRACKER_MAXLINES)

	for i,loot in ipairs(RollTrackerClassicLoot) do
		RTC_LootList_AddItem(loot)
	end

end

RTC_UpdateLootList=function()

	RollTrackerClassicLootFrameStatusText:SetText(string.format(L["MsgNbLoots"],#RollTrackerClassicLoot))
	
	if #RollTrackerClassicLoot==0 and #lootUndo>0 then
		RollTrackerClassicLootFrameClearButton:SetText(L["BtnUndo"])
	else
		RollTrackerClassicLootFrameClearButton:SetText(L["BtnClear"])	
	end	
end
	
function RollTrackerClassic_ScrollLootList(self,delta)
	self:SetScrollOffset(self:GetScrollOffset() + delta*5);
	self:ResetAllFadeTimes()
end
function RollTrackerClassic_ScrollCSVList(self,delta)
	self:SetScrollOffset(self:GetScrollOffset() + delta*5);
	self:ResetAllFadeTimes()
end

RTC_CSVList_AddItem = function(loot)

	CSV_Transfer_Table["%%"]="%%"
	CSV_Transfer_Table["%name%"]=loot.name
	CSV_Transfer_Table["%class%"]=loot.class
	CSV_Transfer_Table["%timestamp%"]=loot.timestamp
	CSV_Transfer_Table["%dd%"]=date("%d",loot.timestamp)
	CSV_Transfer_Table["%mm%"]=date("%m",loot.timestamp)
	CSV_Transfer_Table["%yy%"]=date("%y",loot.timestamp)
	CSV_Transfer_Table["%HH%"]=date("%H",loot.timestamp)
	CSV_Transfer_Table["%MM%"]=date("%M",loot.timestamp)
	CSV_Transfer_Table["%SS%"]=date("%S",loot.timestamp)
	CSV_Transfer_Table["%iname%"]=loot.itemName
	CSV_Transfer_Table["%irarity%"]=loot.itemRarity
	CSV_Transfer_Table["%iraritytxt%"]=_G["ITEM_QUALITY".. loot.itemRarity .."_DESC"]
	CSV_Transfer_Table["%ilevel%"]=loot.itemLevel
	CSV_Transfer_Table["%iid%"]=string.match(loot.itemLink,"item:(.-):")
	CSV_Transfer_Table["%iprice%"]=loot.itemSellPrice
	CSV_Transfer_Table["%icount%"]=loot.ItemCount or 1
	CSV_Transfer_Table["%itype%"]=loot.itemType or 99
	CSV_Transfer_Table["%itypetxt%"]=GetItemClassInfo(loot.itemType or 99) or ""
		
	
	local text= string.gsub(RollTrackerClassicDB.LootTracker.CSVexport,"%%%w+%%",CSV_Transfer_Table)	
	
	RollTrackerClassicCSVFrame_MessageFrame:AddMessage( text,.8,.8,.8)
end

RTC_CSVListInit=function()

	RollTrackerClassicCSVFrame_MessageFrame:SetFading(false);
	RollTrackerClassicCSVFrame_MessageFrame:SetFontObject(GameFontNormalSmall);
	RollTrackerClassicCSVFrame_MessageFrame:SetJustifyH("LEFT");
	RollTrackerClassicCSVFrame_MessageFrame:SetHyperlinksEnabled(true);
	RollTrackerClassicCSVFrame_MessageFrame:SetTextCopyable(true);
	RollTrackerClassicCSVFrame_MessageFrame:Clear()
	RollTrackerClassicCSVFrame_MessageFrame:SetMaxLines(LOOTTRACKER_MAXLINES)
	
	for i,loot in ipairs(RollTrackerClassicLoot) do
		RTC_CSVList_AddItem(loot)
	end

end


	
	